@@ -1,5 +1,4 @@
-use crate::atomic::ArcBool;
-use crate::{atomic, PortalWorkerPair};
+use crate::PortalWorkerPair;
 use ockam_core::async_trait;
 use ockam_core::compat::net::SocketAddr;
 use ockam_core::{Address, Processor, Result, Route};
@@ -11,7 +10,6 @@ use tracing::debug;
 pub(crate) struct TcpInletListenProcessor {
     inner: TcpListener,
     onward_route: Route,
-    run: ArcBool,
 }
 
 impl TcpInletListenProcessor {
@@ -19,7 +17,6 @@ impl TcpInletListenProcessor {
         ctx: &Context,
         onward_route: Route,
         addr: SocketAddr,
-        run: ArcBool,
     ) -> Result<Address> {
         let waddr = Address::random(0);
 
@@ -30,7 +27,6 @@ impl TcpInletListenProcessor {
         let processor = Self {
             inner,
             onward_route,
-            run,
         };
 
         ctx.start_processor(waddr.clone(), processor).await?;
@@ -44,15 +40,9 @@ impl Processor for TcpInletListenProcessor {
     type Context = Context;
 
     async fn process(&mut self, ctx: &mut Self::Context) -> Result<bool> {
-        // FIXME: see ArcBool future note
+        let (stream, peer) = self.inner.accept().await.map_err(TransportError::from)?;
+        PortalWorkerPair::new_inlet(ctx, stream, peer, self.onward_route.clone()).await?;
 
-        if atomic::check(&self.run) {
-            let (stream, peer) = self.inner.accept().await.map_err(TransportError::from)?;
-            PortalWorkerPair::new_inlet(ctx, stream, peer, self.onward_route.clone()).await?;
-
-            Ok(true)
-        } else {
-            Ok(false)
-        }
+        Ok(true)
     }
 }
